@@ -12,7 +12,6 @@ from scipy.stats import scoreatpercentile
 from sklearn.isotonic import IsotonicRegression
 import logging
 import warnings
-import re
 warnings.formatwarning = lambda msg, *args, **kw: str(msg) + '\n'
 logger = logging.getLogger(__name__)
 SEED = 42
@@ -119,11 +118,23 @@ def keywithmaxval(d):
     return k[v.index(max(v))]
 
 
-def add_protein_groups(df, df_ms1_path):
+def add_protein_groups(df, df_ms1_path, sort_random=True):
+    # input: df - pandas dataframe which contains columns "dbname" with protein names and
+    #        "peptides set" with set of peptide sequences belong to this protein and identified in MS/MS analysis.
+    #        df_ms1_path - path to the table *_proteins_full_noexclusion.tsv which is output by DirectMS1 analysis.
+    #        sort_random - if True, the proteins with same scores are chosen randomly. Otherwise, they are chosen
+    #        in alphabetical order.
+    # output: pandas dataframe with new columns "groupleader" (True for protein group leaders) and
+    #         "all proteins" (list of proteins belong to protein group and separeted by ';')
+
     pept_prots = defaultdict(set)
     prot_prots = defaultdict(set)
     prot_pepts = dict()
-    for peptides, dbname in df.sample(frac=1).reset_index(drop=True)[['peptides set', 'dbname']].values:
+    if not sort_random:
+        iter_list = df.sort_values(by='dbname').reset_index(drop=True)[['peptides set', 'dbname']].values
+    else:
+        iter_list = df.sample(frac=1).reset_index(drop=True)[['peptides set', 'dbname']].values
+    for peptides, dbname in iter_list:
         prot_pepts[dbname] = peptides
         for peptide in peptides:
             pept_prots[peptide].add(dbname)
@@ -132,7 +143,7 @@ def add_protein_groups(df, df_ms1_path):
             prot_prots[dbname].update(prots)
     prot_pepts_count = dict()
     prot_pepts_count2 = dict()
-    
+
     if not df_ms1_path:
         for k, v in prot_pepts.items():
             prot_pepts_count[k] = len(v)
@@ -157,7 +168,7 @@ def add_protein_groups(df, df_ms1_path):
                 prot_pepts_count[k] -= 1
                 prot_pepts_count2[k] -= 1
             del pept_prots[pep]
-            
+
         for k, v in list(prot_pepts_count2.items()):
             if v == 0:
                 del prot_pepts_count[k]
@@ -187,6 +198,7 @@ def process_fasta(df, path_to_fasta, decoy_prefix, decoy_infix=False):
                 lambda x: x['sequence'] if x['sequence'] else protsS.get(
                     x['dbname'].replace(decoy_infix, ''), protsS.get(x['dbname'].split(' ')[0].replace(decoy_infix, ''), '')),
                 axis=1)
+
     return df
 
 
@@ -312,9 +324,7 @@ def is_decoy(proteins, decoy_prefix, decoy_infix=False):
         return all(decoy_infix in z for z in proteins)
 
 
-def is_group_specific(proteins, group_prefix, group_infix, group_regex, decoy_prefix, decoy_infix=None):
-    if group_regex:
-        return all(re.search(group_regex, z) for z in proteins)
+def is_group_specific(proteins, group_prefix, group_infix, decoy_prefix, decoy_infix=None):
     if group_infix:
         return all(group_infix in z for z in proteins)
     if not decoy_infix:
@@ -562,14 +572,14 @@ def get_Y_array(df):
     return df.loc[:, 'decoy1'].values.astype(float)
 
 
-def variant_peptides(allowed_peptides, group_prefix, group_infix, group_regex):
+def variant_peptides(allowed_peptides, group_prefix, group_infix):
     if allowed_peptides:
         with open(allowed_peptides) as f:
             allowed_peptides = set(pseq.strip().split()[0] for pseq in f)
     else:
         allowed_peptides = None
 
-    return allowed_peptides, group_prefix, group_infix, group_regex
+    return allowed_peptides, group_prefix, group_infix
 
 
 def filename(outfolder, outbasename, ftype):
@@ -683,16 +693,16 @@ def calc_qvals(df, ratio):
 
 
 _columns_to_output = {
-    'psm_full': ['peptide', 'length', 'spectrum', 'q', 'q_uncorrected', 'ML score', 'modifications', 'assumed_charge',
+    'psm_full': ['peptide', 'length', 'spectrum', 'file', 'q', 'q_uncorrected', 'ML score', 'modifications', 'assumed_charge',
          'num_missed_cleavages', 'num_tol_term', 'peptide_next_aa',
          'peptide_prev_aa', 'calc_neutral_pep_mass', 'massdiff_ppm', 'massdiff_int', 'RT exp', 'RT pred',
          'RT diff', 'protein', 'protein_descr', 'decoy', 'decoy1', 'decoy2', 'PEP',
          'MS1Intensity', 'ISOWIDTHDIFF'],
-    'psm': ['peptide', 'length', 'spectrum', 'q', 'q_uncorrected','ML score', 'modifications', 'assumed_charge',
-         'num_missed_cleavages', 'num_tol_term', 'peptide_next_aa',
+    'psm': ['peptide', 'length', 'spectrum', 'file', 'q', 'q_uncorrected','ML score', 'modifications', 'modified_peptide',
+         'assumed_charge', 'num_missed_cleavages', 'num_tol_term', 'peptide_next_aa',
          'peptide_prev_aa', 'calc_neutral_pep_mass', 'massdiff_ppm', 'massdiff_int', 'RT exp', 'RT pred',
          'protein', 'protein_descr', 'decoy', 'PEP', 'MS1Intensity', 'ISOWIDTHDIFF', 'PIF'],
-    'peptide': ['peptide', '#PSMs', 'length', 'spectrum', 'q', 'q_uncorrected', 'ML score', 'modifications',
+    'peptide': ['peptide', '#PSMs', 'length', 'spectrum', 'file', 'q', 'q_uncorrected', 'ML score', 'modifications',
          'assumed_charge', 'num_missed_cleavages', 'num_tol_term', 'peptide_next_aa',
          'peptide_prev_aa', 'calc_neutral_pep_mass', 'massdiff_ppm', 'massdiff_int', 'RT exp',
          'RT pred', 'protein', 'protein_descr', 'decoy', 'PEP', 'MS1Intensity', 'ISOWIDTHDIFF', 'PIF'],
